//
// Copyright (c) 2023-2024 Fas Xmut (fasxmut at protonmail dot com)
//
// Distributed under the Boost Software License, Version 1.0. (See accompanying
// file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
//

#ifndef __nirtcpp_core_hpp__
#define __nirtcpp_core_hpp__

/**
\mainpage Nirtcpp core layer API documentation

\section description Nirtcpp Engine

Nirtcpp is a high-performance c++ graphics engine, it can be used in 3D graphics, 2D graphics, 3D/2D applications, and as game engine. Nirtcpp forks irrlicht, most of nirtcpp features are the same as or similar to irrlicht features, but with some differences, such as nirtcpp implemented an sfml device backend.

\section example c++ example

\code

#include <nirtcpp/core.hpp>

int main()
{
	nirt::NirtcppDevice * device = nirt::createDevice(
		nirt::video::EDT_BURNINGSVIDEO,	// Select Video Driver
		nirt::core::dimension2du{1280, 720},	// Window Dimension or video mode in fullscrren mode.
		32,	// Bits per pixel, setting is only useful in fullscreen mode.
		false,	// Enable fullscreen?
		true,	// Enable stencilbuffer?
		false,	// Enable vsync?
		nullptr	// Event Receiver
	);

	nirt::video::IVideoDriver * video = device->getVideoDriver();
	nirt::scene::ISceneManager * scene = device->getSceneManager();
	nirt::gui::IGUIEnvironment * gui  = device->getGUIEnvironment();

	while (device->run())
	{
		if (device->isWindowActive())
		{
			video->beginScene(
				nirt::video::ECBF_COLOR | nirt::vieo::ECBF_DEPTH,
				nirt::video::SColor{0xff3277af}
			);
			scene->drawAll();
			gui->drawAll();
			video->endScene();
		}
		else
		{
			device->yield();
		}
	}

	device->drop();
}

\endcode

\section links Links

Home:
<a href="https://cppfx.xyz/nirtcpp">https://cppfx.xyz/nirtcpp</a>
*/

#include <nirtcpp/core/engine.hpp>

#endif	// __nirtcpp_core_hpp__

