// Copyright (C) 2002-2012 Nikolaus Gebhardt
// This file is part of the "Irrlicht Engine".
// For conditions of distribution and use, see copyright notice in nirtcpp/nirtcpp.hpp

#ifndef NIRT_I_PARTICLE_GRAVITY_AFFECTOR_HPP_INCLUDED
#define NIRT_I_PARTICLE_GRAVITY_AFFECTOR_HPP_INCLUDED

#include <nirtcpp/core/engine/IParticleAffector.hpp>

namespace nirt
{
namespace scene
{

//! A particle affector which applies gravity to particles.
class IParticleGravityAffector : public IParticleAffector
{
public:

	//! Set the time in milliseconds when the gravity force is totally lost
	/** At that point the particle does not move any more. */
	virtual void setTimeForceLost( f32 timeForceLost ) = 0;

	//! Set the direction and force of gravity in all 3 dimensions.
	virtual void setGravity( const core::vector3df& gravity ) = 0;

	//! Get the time in milliseconds when the gravity force is totally lost
	virtual f32 getTimeForceLost() const = 0;

	//! Get the direction and force of gravity.
	virtual const core::vector3df& getGravity() const = 0;

	//! Get emitter type
	virtual E_PARTICLE_AFFECTOR_TYPE getType() const override { return EPAT_GRAVITY; }
};

} // end namespace scene
} // end namespace nirt

#endif // NIRT_I_PARTICLE_GRAVITY_AFFECTOR_HPP_INCLUDED
