// Copyright (C) 2002-2012 Nikolaus Gebhardt
// This file is part of the "Irrlicht Engine".
// For conditions of distribution and use, see copyright notice in nirtcpp/nirtcpp.hpp

#ifndef S_EXPOSED_VIDEO_DATA_HPP_INCLUDED
#define S_EXPOSED_VIDEO_DATA_HPP_INCLUDED

// forward declarations for internal pointers
class IDirect3D9;
class IDirect3DDevice9;
class IDirect3D8;
class IDirect3DDevice8;

namespace nirt
{
namespace video
{

//! class for holding data describing a driver and operating system specific data.
/** This data can be retrieved by IVideoDriver::getExposedVideoData(). Use this with caution.
This only should be used to make it possible to extend the engine easily without
modification of its source. Note that this class does not contain any valid data, if
you are using the software or the null device.
*/
class SExposedVideoData
{
public:
	SExposedVideoData() {OpenGLWin32.HDc=0; OpenGLWin32.HRc=0; OpenGLWin32.HWnd=0;}
	explicit SExposedVideoData(void* Window) {OpenGLWin32.HDc=0; OpenGLWin32.HRc=0; OpenGLWin32.HWnd=Window;}

public:
	class SD3D9
	{
	public:
		//! Pointer to the IDirect3D9 interface
		IDirect3D9* D3D9;

		//! Pointer to the IDirect3DDevice9 interface
		IDirect3DDevice9* D3DDev9;

		//! Window handle.
		/** Get with for example HWND h = reinterpret_cast<HWND>(exposedData.D3D9.HWnd) */
		void* HWnd;
	};

public:
	class SOpenGLWin32
	{
	public:
		//! Private GDI Device Context.
		/** Get if for example with: HDC h = reinterpret_cast<HDC>(exposedData.OpenGLWin32.HDc) */
		void* HDc;

		//! Permanent Rendering Context.
		/** Get if for example with: HGLRC h = reinterpret_cast<HGLRC>(exposedData.OpenGLWin32.HRc) */
		void* HRc;

		//! Window handle.
		/** Get with for example with: HWND h = reinterpret_cast<HWND>(exposedData.OpenGLWin32.HWnd) */
		void* HWnd;
	};

public:
	class SOpenGLLinux
	{
	public:
		// XWindow handles
		void* X11Display;
		void* X11Context;
		unsigned long X11Window;
		unsigned long GLXWindow;
	};

public:
    class SOpenGLOSX
    {
    public:
        //! The NSOpenGLContext object.
        void* Context;

        //! The NSWindow object.
        void* Window;
    };

public:
	union
	{
		SD3D9 D3D9;
		SOpenGLWin32 OpenGLWin32;
		SOpenGLLinux OpenGLLinux;
        SOpenGLOSX OpenGLOSX;
	};
};

} // end namespace video
} // end namespace nirt

#endif
