// Copyright (C) 2002-2012 Nikolaus Gebhardt
// This file is part of the "Irrlicht Engine".
// For conditions of distribution and use, see copyright notice in nirtcpp/nirtcpp.hpp

#ifndef NIRT_C_IMAGE_LOADER_PCX_HPP_INCLUDED
#define NIRT_C_IMAGE_LOADER_PCX_HPP_INCLUDED

#include <nirtcpp/core/engine/NirtCompileConfig.hpp>

#include <nirtcpp/core/engine/IImageLoader.hpp>

namespace nirt
{
namespace video
{

#if defined(_NIRT_COMPILE_WITH_PCX_LOADER_) || defined(_NIRT_COMPILE_WITH_PCX_WRITER_)

// byte-align classes
#include <nirtcpp/core/engine/irrpack.hpp>

	class SPCXHeader
	{
	public:
		u8	Manufacturer;
		u8	Version;
		u8	Encoding;
		u8	BitsPerPixel;
		u16	XMin;
		u16	YMin;
		u16	XMax;
		u16	YMax;
		u16	HorizDPI;
		u16	VertDPI;
		u8	Palette[48];
		u8	Reserved;
		u8	Planes;
		u16	BytesPerLine;
		u16	PaletteType;
		u16	HScrSize;
		u16	VScrSize;
		u8	Filler[54];
	} PACK_CLASS;


// Default alignment
#include <nirtcpp/core/engine/irrunpack.hpp>

#endif // compile with loader or writer

#ifdef _NIRT_COMPILE_WITH_PCX_LOADER_

/*!
	Image Loader for Windows PCX bitmaps.
	This loader was written and sent in by Dean P. Macri. I modified
	only some small bits of it.
*/
class CImageLoaderPCX : public IImageLoader
{
public:

	//! constructor
	CImageLoaderPCX();

	//! returns true if the file maybe is able to be loaded by this class
	//! based on the file extension (e.g. ".tga")
	virtual bool isALoadableFileExtension(const io::path& filename) const override;

	//! returns true if the file maybe is able to be loaded by this class
	virtual bool isALoadableFileFormat(io::IReadFile* file) const override;

	//! creates a surface from the file
	virtual IImage* loadImage(io::IReadFile* file) const override;

};

#endif // compile with loader

} // end namespace video
} // end namespace nirt

#endif
