// Copyright (C) 2002-2012 Nikolaus Gebhardt
// This file is part of the "Irrlicht Engine".
// For conditions of distribution and use, see copyright notice in nirtcpp/nirtcpp.hpp
//
// I (Nikolaus Gebhardt) did some few changes to Jonas Petersen's original loader:
// - removed setTexturePath() and replaced with the ISceneManager::getStringParameter()-stuff.
// - added EAMT_LMTS enumeration value
// Thanks a lot to Jonas Petersen for his work
// on this and that he gave me his permission to add it into Nirtcpp.
/*

CLMTSMeshFileLoader.hpp

LMTSMeshFileLoader
Written by Jonas Petersen (a.k.a. jox)

Version 1.5 - 15 March 2005

*/

#ifndef NIRT_C_LMTS_MESH_FILE_LOADER_HPP_INCLUDED
#define NIRT_C_LMTS_MESH_FILE_LOADER_HPP_INCLUDED

#include <nirtcpp/core/engine/IMeshLoader.hpp>
#include <nirtcpp/core/engine/SMesh.hpp>
#include <nirtcpp/core/engine/IFileSystem.hpp>
#include <nirtcpp/core/engine/IVideoDriver.hpp>

namespace nirt
{
namespace scene
{

class CLMTSMeshFileLoader : public IMeshLoader
{
public:

	CLMTSMeshFileLoader(io::IFileSystem* fs,
		video::IVideoDriver* driver, io::IAttributes* parameters);

	virtual ~CLMTSMeshFileLoader();

	virtual bool isALoadableFileExtension(const io::path& filename) const override;

	virtual IAnimatedMesh* createMesh(io::IReadFile* file) override;

private:
	void constructMesh(SMesh* mesh);
	void loadTextures(SMesh* mesh);
	void cleanup();

// byte-align classes
#include <nirtcpp/core/engine/irrpack.hpp>

	class SLMTSHeader
	{
	public:
		u32 MagicID;
		u32 Version;
		u32 HeaderSize;
		u16 TextureCount;
		u16 SubsetCount;
		u32 TriangleCount;
		u16 SubsetSize;
		u16 VertexSize;
	} PACK_CLASS;

	class SLMTSTextureInfoEntry
	{
	public:
		c8 Filename[256];
		u16 Flags;
	} PACK_CLASS;

	class SLMTSSubsetInfoEntry
	{
	public:
		u32 Offset;
		u32 Count;
		u16 TextID1;
		u16 TextID2;
	} PACK_CLASS;

	class SLMTSTriangleDataEntry
	{
	public:
		f32 X;
		f32 Y;
		f32 Z;
		f32 U1;
		f32 V1;
		f32 U2;
		f32 V2;
	} PACK_CLASS;

// Default alignment
#include <nirtcpp/core/engine/irrunpack.hpp>

	SLMTSHeader Header;
	SLMTSTextureInfoEntry* Textures;
	SLMTSSubsetInfoEntry* Subsets;
	SLMTSTriangleDataEntry* Triangles;

	io::IAttributes* Parameters;
	video::IVideoDriver* Driver;
	io::IFileSystem* FileSystem;
	bool FlipEndianess;
};

} // end namespace scene
} // end namespace nirt

#endif // NIRT_C_LMTS_MESH_FILE_LOADER_HPP_INCLUDED
